import os
import sys
import yaml

from ..src import backup_manager

CONFIG_FILE_LIST = [
  '.backup-client.cfg',
  '~/.backup-client.cfg',
  '/etc/backup-client.cfg',
]

manager_impl_dict = {
    'rsync': backup_manager.RsyncBackupManager,
    'rdiff-backup': backup_manager.RdiffBackupManager,
}

def main():
    for filename in CONFIG_FILE_LIST:
        if os.path.exists(filename):
            with open(filename) as f:
                config = yaml.load(f.read())
            break
    else:
        print('No configuration file found')
        sys.exit(1)

    try:
        backup_manager = manager_impl_dict[config['copy_tool']](
            config['copy_tool'],
            config['path_list'],
            config['remote_path'],
            config.get('extra_args', []),
            config.get('dry_run', False),
        )
    except KeyError as e:
        print('Option %s not found in configuration file %s' % (e, filename))
        sys.exit(1)
    except AttributeError as e:
        print(e)
        sys.exit(1)

    backup_manager.start()
