import os
import subprocess
import sys

class BackupManager:
    def __init__(self, copy_binary, path_list, remote_path, extra_args, dry_run):
        self.copy_binary = copy_binary
        self.remote_path = remote_path
        self.extra_args = extra_args
        self.dry_run = dry_run
        self.path_list = []

        for path in path_list:
            path = os.path.abspath(
                    os.path.expanduser(
                        os.path.expandvars(path)))
            if not os.path.exists(path):
                raise AttributeError('Path %s doesn\'t exist' % path)
            self.path_list.append(path)


    def _build_command_list(self):
        raise NotImplementedError()


    def start(self):
        command_list = self._build_command_list()
        for command in command_list:
            print('Running : %s' % ' '.join(command))
            subprocess.call(
                    command,
                    stdout=sys.stdout,
            )


class RsyncBackupManager(BackupManager):

    def _build_command_list(self):
        command_list = []
        hostname = os.uname().nodename
        short_option_list = '-az'

        if self.dry_run:
            short_option_list += 'n'

        for path in self.path_list:
            command_list.append((
                self.copy_binary,
                short_option_list,
                '--relative',
                *self.extra_args,
                path,
                self.remote_path + '/' + hostname,
            ))

        return command_list


class RdiffBackupManager(BackupManager):

    def _build_command_list(self):
        command_list = []
        hostname = os.uname().nodename

        if self.dry_run:
            self.extra_args.append('--compare-hash')

        for path in self.path_list:
            command_list.append((
                self.copy_binary,
                '--create-full-path',
                *self.extra_args,
                path,
                self.remote_path + '/' + hostname,
            ))

        return command_list
